# -*- coding: utf-8 -*-
# Generated by Django 1.11.23 on 2019-09-01 17:33
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0029_payment_restrictions'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='leprikonsite',
            name='old_registration_agreement',
        ),
        migrations.RemoveField(
            model_name='subject',
            name='old_registration_agreement',
        ),
        migrations.RemoveField(
            model_name='subjecttype',
            name='old_registration_agreement',
        ),
    ]
