{% for group, options, index in widget.optgroups %}{% if group %}
<strong>{{ group }}</strong>{% endif %}{% for option in options %}
{% include option.template_name with widget=option %}{% endfor %}{% endfor %}
